from tkinter import ttk
from tkinter import Tk
from tkinter import Button, Label
from .data import Message
from src import sendkeys
from typing import Callable
from src.data import log
import sv_ttk


class Gui(Tk):
    def __init__(self, parent=None) -> None:
        Tk.__init__(self, parent)
        sv_ttk.set_theme("dark")
        self.title("Uber Trader")
        # self.geometry("300x200+10+20")
        self.overrideredirect(1)  # remove border
        self.wm_withdraw()  # Hide window
        self.attributes('-topmost', True)  # always on top
        # style = ttk.Style()
        # style.configure("BW.TLabel", foreground="white", background="black")
        self.tab_control = ttk.Notebook(self, style='BW.TLabel')
        self.tab_control.pack(expand=1, fill='both')
        self.tab_control.bind("<ButtonPress-1>", self.start_move)
        self.tab_control.bind("<ButtonRelease-1>", self.stop_move)
        self.tab_control.bind("<B1-Motion>", self.do_move)
        self.tabs = dict()

    def add_tab(self, number: int, message: Message) -> None:
        self.wm_deiconify()
        if message.user not in self.tabs:
            self.tabs[message.user] = {'frame': ttk.Frame(self.tab_control)}
            self.tabs[message.user]['items'] = {}
        frame = self.tabs[message.user]['frame']
        if message.trade is not None:
            if message.trade.item not in self.tabs[message.user]['items']:
                row = len(self.tabs[message.user]['items'])
                self.tabs[message.user]['items'][message.trade.item] = {}
                self.tabs[message.user]['items'][message.trade.item]['row'] = row
                self.tabs[message.user]['items'][message.trade.item]['message'] =  message
                ttk.Label(frame, text=message.trade.item).grid(column=0, row=row)
                Label(frame, text=f'{message.trade.amount} {message.trade.currency}').grid(column=1, row=row)
                self.add_button(tab=frame, text='Inv', callback=self.inv_callback, message=message).grid(column=2, row=row)
                self.add_button(tab=frame, text='Trade', callback=self.trade_callback, message=message).grid(column=3, row=row)
                self.add_button(tab=frame, text='Ty', callback=self.ty_callback, message=message).grid(column=4, row=row)
                self.add_button(tab=frame, text='Wait', callback=self.wait_callback, message=message).grid(column=5, row=row)
                self.add_button(tab=frame, text='X', callback=self.destroy_tab, message=message).grid(column=6, row=row)
                self.tab_control.add(frame, text=message.user)
        else:
            log.warning(f'Trade in message "{str(message)}" is None')

    def add_button(self, tab: ttk.Frame, text: str, callback: Callable, message: Message) -> Button:
        return Button(tab, text=text, command=lambda: callback(message, tab))

    def start_move(self, event):
        self.x = event.x
        self.y = event.y

    def stop_move(self, event):
        self.x = None
        self.y = None

    def do_move(self, event):
        deltax = event.x - self.x
        deltay = event.y - self.y
        x = self.winfo_x() + deltax
        y = self.winfo_y() + deltay
        self.geometry(f"+{x}+{y}")

    def inv_callback(self, message: Message, tab: ttk.Frame):
        sendkeys.invite(message=message)
        sendkeys.send_to_format(type='pickup', message=message)

    def ty_callback(self, message: Message, tab: ttk.Frame):
        sendkeys.send_to_format(type='ty', message=message)
        sendkeys.kick(message=message)
        self.destroy_tab(message=message, tab=tab)

    def trade_callback(self, message: Message, tab: ttk.Frame):
        sendkeys.trade(message=message)

    def wait_callback(self, message: Message, tab: ttk.Frame):
        sendkeys.send_to_format(type='wait', message=message)

    def destroy_tab(self, message: Message, tab: ttk.Frame) -> None:
        if message.user in self.tabs:
            if message.trade.item in self.tabs[message.user]['items']:
                # Delete the item
                row = self.tabs[message.user]['items'][message.trade.item]['row']
                for widget in self.tabs[message.user]['frame'].grid_slaves(row=row):
                    widget.destroy()
                del self.tabs[message.user]['items'][message.trade.item]
            if len(self.tabs[message.user]['items']) == 0:
                # If no item from this user is left, then also delete the tab
                del self.tabs[message.user]
                tab.destroy()
        if not self.tabs:
            self.wm_withdraw()
